@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2018 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -57,7 +57,7 @@ sub source {
 	return ();
     }
     my $cwd = getcwd();
-    chdir($ptable->{cwd});
+    chdir("/proc/$pid/root/$ptable->{cwd}");
 
     # get original ARGV
     (my $bin, local @ARGV) = nr_parse_cmd($pid);
@@ -97,13 +97,20 @@ sub source {
 sub files {
     my $self = shift;
     my $pid = shift;
+    my $cache = shift;
     my $ptable = nr_ptable_pid($pid);
     unless($ptable->{cwd}) {
 	print STDERR "$LOGPREF #$pid: could not get current working directory, skipping\n" if($self->{debug});
 	return ();
     }
     my $cwd = getcwd();
-    chdir($ptable->{cwd});
+    chdir("/proc/$pid/root/$ptable->{cwd}");
+
+    # skip the process if the cwd is unreachable (i.e. due to mnt ns)
+    unless(getcwd()) {
+	print STDERR "$LOGPREF #$pid: process cwd is unreachable\n" if($self->{debug});
+	return ();
+    }
 
     # get original ARGV
     (my $bin, local @ARGV) = nr_parse_cmd($pid);
@@ -128,13 +135,21 @@ sub files {
 	print STDERR "$LOGPREF #$pid: could not get a source file, skipping\n" if($self->{debug});
 	return ();
     }
-    my $src = $ARGV[0];
+    my $src = abs_path ($ARGV[0]);
     unless(-r $src && -f $src) {
 	chdir($cwd);
 	print STDERR "$LOGPREF #$pid: source file not found, skipping\n" if($self->{debug});
 	print STDERR "$LOGPREF #$pid:  reduced ARGV: ".join(' ', @ARGV)."\n" if($self->{debug});
 	return ();
     }
+    print STDERR "$LOGPREF #$pid: source=$src\n" if($self->{debug});
+
+    # use cached data if avail
+    if(exists($cache->{files}->{(__PACKAGE__)}->{$src})) {
+	chdir($cwd);
+	print STDERR "$LOGPREF #$pid: use cached file list\n" if($self->{debug});
+	return %{ $cache->{files}->{(__PACKAGE__)}->{$src} };
+    }
 
     # prepare include path environment variable
     my %e = nr_parse_env($pid);
@@ -147,17 +162,25 @@ sub files {
     }
 
     @Module::ScanDeps::IncludeLibs = (exists($opts{I}) ? ($opts{I}) : ());
-    my $href = scan_deps(
-	files => [$src],
-	recurse => 1,
-    );
+    my $href;
+    {
+	# Silence warnings of Module::ScanDeps for dynamic loaded modules (github issue #41)
+	local $SIG{__WARN__} = sub { };
+
+	$href = scan_deps(
+	    files => [$src],
+	    recurse => 1,
+	    );
+    }
 
     my %ret = map {
-	my $stat = nr_stat($href->{$_}->{file});
+	my $stat = nr_stat("/proc/$pid/root/$href->{$_}->{file}");
 	$href->{$_}->{file} => ( defined($stat) ? $stat->{ctime} : undef );
     } keys %$href;
 
     chdir($cwd);
+
+    $cache->{files}->{(__PACKAGE__)}->{$src} = \%ret;
     return %ret;
 }
 
