@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2018 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -85,7 +85,7 @@ sub source {
 	return ();
     }
     my $cwd = getcwd();
-    chdir($ptable->{cwd});
+    chdir("/proc/$pid/root/$ptable->{cwd}");
 
     # get original ARGV
     (my $bin, local @ARGV) = nr_parse_cmd($pid);
@@ -125,13 +125,20 @@ sub source {
 sub files {
     my $self = shift;
     my $pid = shift;
+    my $cache = shift;
     my $ptable = nr_ptable_pid($pid);
     unless($ptable->{cwd}) {
 	print STDERR "$LOGPREF #$pid: could not get current working directory, skipping\n" if($self->{debug});
 	return ();
     }
     my $cwd = getcwd();
-    chdir($ptable->{cwd});
+    chdir("/proc/$pid/root/$ptable->{cwd}");
+
+    # skip the process if the cwd is unreachable (i.e. due to mnt ns)
+    unless(getcwd()) {
+	print STDERR "$LOGPREF #$pid: process cwd is unreachable\n" if($self->{debug});
+	return ();
+    }
 
     # get original ARGV
     (my $bin, local @ARGV) = nr_parse_cmd($pid);
@@ -163,6 +170,13 @@ sub files {
 	print STDERR "$LOGPREF #$pid:  reduced ARGV: ".join(' ', @ARGV)."\n" if($self->{debug});
 	return ();
     }
+    print STDERR "$LOGPREF #$pid: source=$src\n" if($self->{debug});
+
+    # use cached data if avail
+    if(exists($cache->{files}->{(__PACKAGE__)}->{$src})) {
+	print STDERR "$LOGPREF #$pid: use cached file list\n" if($self->{debug});
+	return %{ $cache->{files}->{(__PACKAGE__)}->{$src} };
+    }
 
     # prepare include path environment variable
     my %e = nr_parse_env($pid);
@@ -176,7 +190,7 @@ sub files {
     
     # get include path
     my $rbread = nr_fork_pipe($self->{debug}, $ptable->{exec}, '-e', 'puts $:');
-    my @path = <$rbread>;
+    my @path = map { "/proc/$pid/root/$_"; } <$rbread>;
     close($rbread);
     chomp(@path);
 
@@ -184,11 +198,13 @@ sub files {
     _scan($self->{debug}, $pid, $src, \%files, \@path);
 
     my %ret = map {
-	my $stat = nr_stat($_);
+	my $stat = nr_stat("/proc/$pid/root/$_");
 	$_ => ( defined($stat) ? $stat->{ctime} : undef );
     } keys %files;
 
     chdir($cwd);
+
+    $cache->{files}->{(__PACKAGE__)}->{$src} = \%ret;
     return %ret;
 }
 
