@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2018 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -105,6 +105,7 @@ sub nr_linux_version_generic($$) {
 
     # fallback trying filename
     $fn =~ s/[^-]*-//;
+    $fn =~ s/\.img$//;
     if($fn =~ /^\d+\.\d+/) {
 	print STDERR "$LOGPREF version from filename: $fn\n" if($debug);
 
@@ -125,11 +126,17 @@ sub nr_kernel_check_real($$) {
 
     die "$LOGPREF Not running on Linux!\n" unless($sysname eq 'Linux');
 
-    my @kfiles = reverse nsort </boot/vmlinu*>;
-    $ui->progress_prep(scalar @kfiles, __ 'Scanning linux images...');
+    my %kfiles = map {
+	$_ => 1,
+    } grep {
+	# filter initrd images
+	(!m@^/boot/init@);
+    } (</boot/vmlinu*>, </boot/*.img>);
+
+    $ui->progress_prep(scalar keys %kfiles, __ 'Scanning linux images...');
 
     my %kernels;
-    foreach my $fn (@kfiles) {
+    foreach my $fn (reverse nsort keys %kfiles) {
 	$ui->progress_step;
 	my $stat = nr_stat($fn);
 
@@ -159,8 +166,7 @@ sub nr_kernel_check_real($$) {
 
 	my $iversion = $verstr;
 	$iversion =~ s/^Linux version //;
-	chomp($iversion);
-	$iversion =~ s/\s.+$//;
+	$iversion =~ s/\s.+$//s;
 	$kernels{$iversion} = (index($verstr, $release) != -1 && index($verstr, $version) != -1);
 
 	print STDERR "$LOGPREF $fn => $verstr [$iversion]".($kernels{$iversion} ? '*' : '')."\n" if($debug);
@@ -172,7 +178,13 @@ sub nr_kernel_check_real($$) {
 	return (NRK_UNKNOWN, %vars);
     }
 
-    ($vars{EVERSION}) = reverse sort { nr_kernel_vcmp($a, $b); } keys %kernels;
+    if(-e "/etc/redhat-release" && !-e "/etc/debian_version") {
+	print STDERR "$LOGPREF using RPM version sorting\n" if($debug);
+	($vars{EVERSION}) = reverse sort { nr_kernel_vcmp_rpm($a, $b); } keys %kernels;
+    }
+    else {
+	($vars{EVERSION}) = reverse sort { nr_kernel_vcmp($a, $b); } keys %kernels;
+    }
     print STDERR "$LOGPREF Expected linux version: $vars{EVERSION}\n" if($debug);
 
     return (NRK_VERUPGRADE, %vars) if($vars{KVERSION} ne $vars{EVERSION});
