@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2018 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -29,7 +29,6 @@ use warnings;
 
 use parent qw(NeedRestart::UI);
 use NeedRestart qw(:ui);
-use Term::ReadKey;
 use Sort::Naturally;
 
 use constant {
@@ -114,6 +113,21 @@ sub announce_ehint {
     dcres( go );
 }
 
+sub announce_ucode {
+    my $self = shift;
+    my %vars = @_;
+    my $templ = 'needrestart/ui-ucode_announce';
+
+    foreach my $k (keys %vars) {
+        dcres( subst($templ, $k, $vars{$k}) );
+    }
+
+    dcres( fset($templ, 'seen', 0) );
+    dcres( settitle('needrestart/ui-ucode_title') );
+    dcres( input('critical', $templ) );
+    dcres( go );
+}
+
 
 sub notice {
     my $self = shift;
@@ -127,6 +141,22 @@ sub notice {
     $self->wprint(\*STDERR, '', $indent, "$out\n");
 }
 
+sub vspace {
+    my $self = shift;
+
+    return unless($self->{verbosity});
+
+    $self->SUPER::vspace(\*STDERR);
+}
+
+
+sub command {
+    my $self = shift;
+    my $out = shift;
+
+    print STDERR "$out\n";
+}
+
 
 sub query_pkgs($$$$$$) {
     my $self = shift;
